<?php
/** 
*
* @package phpBB3
* @version $Id: smartfeed_url.php, v 2.2.27 Production 2015/01/04 mark@phpbbservices.com Exp $
* @copyright (c) Mark D. Hamill (mark@phpbbservices.com, http://phpbbservices.com) 
* @license http://opensource.org/licenses/gpl-2.0.php GNU Public License 
*
*/

/**
* @ignore
*/

// Written by Mark D. Hamill, mark@phpbbservices.com, http://phpbbservices.com
// This software is designed to work with phpBB Version 3.0.12
 
// This is the user interface for the Smartfeed software. Users can use it to create a URL they can use 
// with newsfeed readers like FeedReader or web based personal news aggregators like Google Reader.
// Unlike most RSS newsfeeds this one can dip into member-only and user group permission based
// forums which should not be accessible to the world and return posts in these topics. It handles
// RSS and Atom feeds.

define('IN_PHPBB', true);
$phpbb_root_path = (defined('PHPBB_ROOT_PATH')) ? PHPBB_ROOT_PATH : './';
$phpEx = substr(strrchr(__FILE__, '.'), 1);

include($phpbb_root_path . 'common.' . $phpEx);

if (!defined('SMARTFEED_NO_LIMIT_VALUE'))
{
	include($phpbb_root_path . 'includes/constants_smartfeed.' . $phpEx);
}

// Start session management
$user->session_begin();
$auth->acl($user->data);
$user->setup('mods/smartfeed');

// Smartfeed cannot be used with Apache authentication unless the .htaccess file is modified to allow smartfeed.php to bypass
// Apache authentication. If you have made these changes then set the constant SMARTFEED_APACHE_HTACCESS_ENABLED to true in the ACP interface.
if (($config['auth_method'] == 'apache') && ($config['sf_apache_htaccess_enabled'] != 1))
{
	$msg_text = ($user->data['user_type'] == USER_FOUNDER) ? $user->lang['SMARTFEED_ERROR']['SMARTFEED_APACHE_AUTHENTICATION_WARNING_ADMIN'] : $user->lang['SMARTFEED_ERROR']['SMARTFEED_APACHE_AUTHENTICATION_WARNING_REG'];
	trigger_error($msg_text, E_USER_NOTICE);
}

// Create a full site URL. 
$board_url = generate_board_url() . '/';
$registered_user_mode = false;

// Create a list of required and excluded forum_ids
$required_forum_ids = (isset($config['sf_include_forums']) && strlen(trim($config['sf_include_forums'])) > 0) ? explode(',', $config['sf_include_forums']) : array();
$excluded_forum_ids = (isset($config['sf_exclude_forums']) && strlen(trim($config['sf_exclude_forums'])) > 0) ? explode(',', $config['sf_exclude_forums']) : array();

if ($user->data['is_registered'])
{
	// If the user is registered then great, they can authenticate and see private forums
	$smartfeed_user_id = $user->data['user_id'];
	$user_password = $user->data['user_password'];
	if ($user->data['user_smartfeed_key'])
	{
		$user_smartfeed_key = $user->data['user_smartfeed_key'];
		$encrypted_password = encrypt($user_password, $user_smartfeed_key);
		$encrypted_password_with_ip = encrypt($user_password . '~' . $user->ip, $user_smartfeed_key);
	}
	else
	{
		// Generate a smartfeed encryption key. This is a one time action. It is used to authenticate the user when they call smartfeed.php.
		$user_smartfeed_key = gen_rand_string(32);
		$encrypted_password = encrypt($user_password, $user_smartfeed_key);
		$encrypted_password_with_ip = encrypt($user_password . '~' . $user->ip, $user_smartfeed_key);
		
		// Store the key
		$sql = 'UPDATE ' . USERS_TABLE . "
			   SET user_smartfeed_key = '" . $db->sql_escape($user_smartfeed_key) . "'
			   WHERE user_id = " . $user->data['user_id'];
		$result = $db->sql_query($sql);
	}
	$registered_user_mode = true;
	$template->assign_vars(array('S_IS_GUEST' => false, 'DAY_DEFAULT' => ''));
}
else
{
	// Public (anonymous) users do not need to authenticate so no encrypted passwords are needed
	$smartfeed_user_id = ANONYMOUS;
	$encrypted_password = 'NONE';
	$encrypted_password_with_ip = 'NONE';
	$template->assign_vars(array('S_IS_GUEST' => true, 'DAY_DEFAULT' => 'selected="selected"'));
}

// If IP Authentication is required, hide HTML that solicits whether the user wants to use IP authentication or not ... they have no choice!
if (($config['sf_require_ip_authentication'] == 1) && $registered_user_mode)
{
	$template->assign_vars(array('S_REQUIRED_IP_AUTHENTICATION' => true));	
}
else if ($registered_user_mode)
{
	$template->assign_vars(array(
		'S_REQUIRED_IP_AUTHENTICATION' => false,
		'L_IP_AUTHENTICATION' => $user->lang['SMARTFEED_IP_AUTH']
	));	
}

$allowed_forums = array();
$forum_read_ary = array();

// Get forum read permissions for this user. They are also usually stored in the user_permissions column, but sometimes the field is empty. This always works.
$forum_array = $auth->acl_raw_data_single_user($smartfeed_user_id);
foreach ($forum_array as $key => $value)
{
	foreach ($value as $auth_option_id => $auth_setting)
	{
		if ($auth->acl_get('f_read', $key))
		{
			$forum_read_ary[$key]['f_read'] = 1;
		}
		if ($auth->acl_get('f_list', $key))
		{
			$forum_read_ary[$key]['f_list'] = 1;
		}
	}
}

// Get a list of parent_ids for each forum and put them in an array.
$parent_array = array();
$sql = 'SELECT forum_id, parent_id 
	FROM ' . FORUMS_TABLE . '
	ORDER BY forum_id ASC';
$result = $db->sql_query($sql);
while ($row = $db->sql_fetchrow($result))
{
	$parent_array[$row['forum_id']] = $row['parent_id'];
}
$db->sql_freeresult($result);

if (sizeof($forum_read_ary ) > 0) // This should avoid a PHP Notice
{
	foreach ($forum_read_ary as $forum_id => $allowed)
	{
		if ($auth->acl_get('f_read', $forum_id) && $auth->acl_get('f_list', $forum_id) && check_all_parents($forum_id))
		{
			// Since this user has read access to this forum, add it to the $allowed_forums array
			$allowed_forums[] = (int) $forum_id;
			
			// Also add to $allowed_forums the parents, if any, of this forum. Actually we have to find the parent's parents, etc., going up as far as necessary because 
			// $auth->act_getf does not return the parents for which the user has access, yet parents must be shown are on the interface
			$there_are_parents = true;
			$this_forum_id = (int) $forum_id;
			
			while ($there_are_parents)
			{
				if ($parent_array[$this_forum_id] == 0)
				{
					$there_are_parents = false;
				}
				else
				{
					// Do not add this parent to the list of allowed forums if it is already in the array
					if (!in_array((int) $parent_array[$this_forum_id], $allowed_forums))
					{
						$allowed_forums[] = (int) $parent_array[$this_forum_id];
					} 
					$this_forum_id = (int) $parent_array[$this_forum_id];	// Keep looping...
				}
			}
		}
	}
}

// Get a list of forums as they appear on the main index for this user. For presentation purposes indent them so they show the natural phpBB3 hierarchy.
// Indenting is cleverly handled by nesting <div> tags inside of other <div> tags, and the template defines the relative offset (20 pixels).

if (sizeof($allowed_forums) > 0)

{

	$sql = 'SELECT forum_name, forum_id, parent_id, forum_type
			FROM ' . FORUMS_TABLE . ' 
			WHERE ' . $db->sql_in_set('forum_id', $allowed_forums) . ' AND forum_type <> ' . FORUM_LINK . '
			ORDER BY left_id ASC';
	$result = $db->sql_query($sql);
	
	$template->assign_block_vars('show_forums', array());
	
	$current_level = 0;			// How deeply nested are we at the moment
	$parent_stack = array();	// Holds a stack showing the current parent_id of the forum
	$parent_stack[] = 0;		// 0, the first value in the stack, represents the <div_0> element, a container holding all the categories and forums in the template
	
	while ($row = $db->sql_fetchrow($result))
	{
	
		if ((int) $row['parent_id'] != (int) end($parent_stack) || (end($parent_stack) == 0))
		{
			if (in_array($row['parent_id'], $parent_stack))
			{
				// If parent is in the stack, then pop the stack until the parent is found, otherwise push stack adding the current parent. This creates a </div>
				while ((int) $row['parent_id'] != (int) end($parent_stack))
				{
					array_pop($parent_stack);
					$current_level--;
					// Need to close a category level here
					$template->assign_block_vars('forums', array( 
						'S_DIV_OPEN' => false,
						'S_PRINT' => false));
				}
			}
			else
			{
				// If the parent is not in the stack, then push the parent_id on the stack. This is also a trigger to indent the block. This creates a <div>
				array_push($parent_stack, (int) $row['parent_id']);
				$current_level++;
				// Need to add a category level here
				$template->assign_block_vars('forums', array( 
					'S_DIV_OPEN' => true,
					'CAT_ID' => 'div_' . $row['parent_id'],
					'S_PRINT' => false));
			}
		}
		
		// This section contains logic to handle forums that are either required or excluded by the Administrator
		
		// Is the forum either required or excluded from Smartfeed?
		$required_forum = (in_array((int) $row['forum_id'], $required_forum_ids)) ? true : false;
		$excluded_forum = (in_array((int) $row['forum_id'], $excluded_forum_ids)) ? true : false;
		$forum_disabled = $required_forum || $excluded_forum;
		
		// Markup to visually show required or excluded forums
		if ($required_forum)
		{
			$prefix = '<strong>';
			$suffix = '</strong>';
		}
		else
		{
			if ($excluded_forum)
			{
				$prefix = '<span style="text-decoration:line-through">';
				$suffix = '</span>';
			}
			else
			{
				$prefix = '';
				$suffix = '';
			}
		}
		
		// Markup to indicate whether the checkbox for the forum should be checked or not
		$forum_checked = ($config['sf_all_by_default'] == '1');
		if ($required_forum)
		{
			$forum_checked = true;
		}
		if ($excluded_forum)
		{
			$forum_checked = false;
		}
		
		$element_prefix = ($required_forum || $excluded_forum) ? 'xlt_' : 'elt_'; // 'xlt_' will exclude the element from the check/uncheck form feature
		
		// This code prints the forum or category, which will exist inside the previously created <div> block
		$template->assign_block_vars('forums', array( 
			'FORUM_NAME' => $element_prefix . (int) $row['forum_id'] . '_' . (int) $row['parent_id'],
			'FORUM_PREFIX' => $prefix,
			'FORUM_LABEL' => $row['forum_name'],
			'FORUM_SUFFIX' => $suffix,
			'FORUM_DISABLED' => ($forum_disabled) ? 'disabled="disabled"' : '',
			'FORUM_CHECKED' => ($forum_checked) ? 'checked="checked"' : '',
			'S_PRINT' => true,
			'S_IS_FORUM' => ($row['forum_type'] == FORUM_CAT) ? false : true));	// Switch to display a category different than a forum
		
	}

	$db->sql_freeresult($result);
	
	// Now out of the loop, it is important to remember to close any open <div> tags. Typically there is at least one.
	while ((int) $row['parent_id'] != (int) end($parent_stack))
	{
		array_pop($parent_stack);
		$current_level--;
		// Need to close the <div> tag
		$template->assign_block_vars('forums', array( 
			'S_DIV_OPEN' => false,
			'S_PRINT' => false));
	}

	if ($user->ip == '::1')	// Can happen in local test environment, like XAMPP
	{
		$user->ip = '127.0.0.1';	// Typical default IP for localhost
	}
	
	// If user_smartfeed_ip exists, parse it, otherwise use $user->ip
	if (isset($user->data['user_smartfeed_ip']))
	{
		$user_smartfeed_ip = explode('.', $user->data['user_smartfeed_ip']);
	}
	else
	{
		$user_smartfeed_ip = explode('.', $user->ip);
	}
	$client_ip_parts = explode('.', $user->ip);
	$is_ipV6 = (sizeof($client_ip_parts) == 8) ? true : false;
	$IPV6 = (sizeof($client_ip_parts) == 8) ? 'true' : 'false';
	
	// Handles an improbable PHP Notice problem
	for ($i=0;$i<sizeof($client_ip_parts);$i++)
	{
		if (is_null($client_ip_parts[$i]))
		{
			$client_ip_parts[$i] = 0;
		}
	}
	
	// Set up text for the IP authentication explanation string
	$smartfeed_ip_auth_explain_v4 = sprintf($user->lang['SMARTFEED_IP_AUTHENTICATION_EXPLAIN'], $client_ip_parts[0] . '.' . $client_ip_parts[1] . '.' . $client_ip_parts[2] . '.*', $client_ip_parts[0] . '.' . $client_ip_parts[1] . '.*.*');
	if ($is_ipV6)
	{
		$smartfeed_ip_auth_explain_v6 = sprintf($user->lang['SMARTFEED_IP_AUTHENTICATION_EXPLAIN'], $client_ip_parts[0] . '.' . $client_ip_parts[1] . '.' . $client_ip_parts[2] . '.' . $client_ip_parts[3] . '.' . $client_ip_parts[4] . '.' . $client_ip_parts[5] . '.' . $client_ip_parts[6] . '.*', 	$client_ip_parts[0] . '.' . $client_ip_parts[1] . '.' . $client_ip_parts[2] . '.' . $client_ip_parts[3] . '.' . $client_ip_parts[4] . '.' . $client_ip_parts[5] . '.*.*');
	}

	// Now to output most of the lovely text to the screen
	$template->assign_vars(array(
		'ALL_BY_DEFAULT'			=> ($config['sf_all_by_default'] == '1') ? 'checked="checked"' : '',
		'IP'						=> $user->ip,
		'L_1_HRS_VALUE'				=> SMARTFEED_LAST_1_HOURS_VALUE,
		'L_12_HRS_VALUE'			=> SMARTFEED_LAST_12_HOURS_VALUE,
		'L_15_MIN_VALUE'			=> SMARTFEED_LAST_15_MINUTES_VALUE,
		'L_3_HRS_VALUE'				=> SMARTFEED_LAST_3_HOURS_VALUE,
		'L_30_MIN_VALUE'			=> SMARTFEED_LAST_30_MINUTES_VALUE,
		'L_6_HRS_VALUE'				=> SMARTFEED_LAST_6_HOURS_VALUE,
		'L_BASIC_VALUE'				=> SMARTFEED_BASIC,
		'L_BOARD'					=> SMARTFEED_BOARD,
		'L_COMPACT_VALUE'			=> SMARTFEED_COMPACT,
		'L_COUNT_LIMIT_EXPLAIN_MAX' => ($config['sf_max_items'] == 0) ? $user->lang['SMARTFEED_COUNT_LIMIT_EXPLAIN_BLANK'] : sprintf($user->lang['SMARTFEED_COUNT_LIMIT_EXPLAIN'], $config['sf_max_items']),
		'L_DAY_VALUE'				=> SMARTFEED_LAST_DAY_VALUE,
		'L_HTML_VALUE'				=> SMARTFEED_HTML,
		'L_HTMLSAFE_VALUE'			=> SMARTFEED_HTMLSAFE,
		'L_IPV6'					=> $IPV6,
		'L_LAST_FETCH_VALUE'		=> SMARTFEED_SINCE_LAST_VISIT,
		'L_MONTH_VALUE'				=> SMARTFEED_LAST_MONTH_VALUE,
		'L_NO_LIMIT_VALUE' 			=> SMARTFEED_NO_LIMIT_VALUE,
		'L_POSTDATE_ASCENDING'		=> SMARTFEED_POSTDATE,
		'L_POSTDATE_DESCENDING'		=> SMARTFEED_POSTDATE_DESC,
		'L_QUARTER_VALUE'			=> SMARTFEED_LAST_QUARTER_VALUE,
		'L_SMARTFEED_IP_AUTHENTICATION_EXPLAIN'	=> ($is_ipV6) ? $smartfeed_ip_auth_explain_v6 : $smartfeed_ip_auth_explain_v4,
		'L_SMARTFEED_LIMIT_SET_EXPLAIN'	=> ($config['sf_default_fetch_time_limit'] == '0') ? '' : sprintf($user->lang['SMARTFEED_LIMIT_SET_EXPLAIN'], round(($config['sf_default_fetch_time_limit']/24), 0)),
		'L_SMARTFEED_MAX_SIZE_EXPLAIN' => ($config['sf_max_size'] == '0') ? $user->lang['SMARTFEED_MAX_SIZE_EXPLAIN_BLANK'] : sprintf($user->lang['SMARTFEED_MAX_SIZE_EXPLAIN'], $config['sf_max_size']),
		'L_SMARTFEED_NOT_LOGGED_IN'	=> sprintf($user->lang['SMARTFEED_NOT_LOGGED_IN'], $phpEx, $phpEx),
		'L_STANDARD'				=> SMARTFEED_STANDARD,
		'L_STANDARD_DESC'			=> SMARTFEED_STANDARD_DESC,
		'L_TWO_WEEKS_VALUE'			=> SMARTFEED_LAST_TWO_WEEKS_VALUE,
		'L_WEEK_VALUE'				=> SMARTFEED_LAST_WEEK_VALUE,
		'PWD'						=> $encrypted_password, 
		'PWD_WITH_IP'				=> $encrypted_password_with_ip, 
		'REQUIRED_FORUMS'			=> (sizeof($required_forum_ids) > 0) ? 'true' : 'false',
		'S_ATOM_10_VALUE'			=> $config['sf_atom_10_value'],
		'S_IP_1'					=> $user_smartfeed_ip[0],
		'S_IP_2'					=> $user_smartfeed_ip[1],
		'S_IP_3'					=> $user_smartfeed_ip[2],
		'S_IP_4'					=> $user_smartfeed_ip[3],
		'S_IP_5'					=> (isset($user_smartfeed_ip[4])) ? $user_smartfeed_ip[4] : '',
		'S_IP_6'					=> (isset($user_smartfeed_ip[5])) ? $user_smartfeed_ip[5] : '',
		'S_IP_7'					=> (isset($user_smartfeed_ip[6])) ? $user_smartfeed_ip[6] : '',
		'S_IS_IPV6'					=> $is_ipV6,
		'S_NO_FORUMS'				=> false,
		'S_RSS_10_VALUE'			=> $config['sf_rss_10_value'],
		'S_RSS_20_VALUE'			=> $config['sf_rss_20_value'],
		'S_SMARTFEED_MAX_ITEMS'		=> (intval($config['sf_max_items']) == 0) ? '' : $config['sf_max_items'],
		'S_SMARTFEED_MAX_SIZE'		=> (intval($config['sf_max_size']) == 0) ? '' : $config['sf_max_size'],
		'S_SMARTFEED_PAGE_URL'		=> $config['sf_smartfeed_page_url'],
		'S_SMARTFEED_TITLE'			=> $user->lang['SMARTFEED_TITLE'],
		'S_SMARTFEED_VERSION'		=> $config['sf_version'],
		'UA_PHPEXT'					=> $phpEx,
		'UA_SESSION_ID'				=> $user->data['session_id'],
		'UA_SITE_URL'				=> $board_url,
		'UA_USER_ID'				=> $smartfeed_user_id,
	
		)
	);

}
	
else
	
{

	// No forums to show!
	$template->assign_vars(array(
		'S_NO_FORUMS' => true,
		'L_NO_FORUMS_MESSAGE' =>  $user->lang['SMARTFEED_NO_FORUMS_AVAILABLE']));
	
}

page_header($user->lang['SMARTFEED_TITLE']);

$template->set_filenames(array(
	'body' => 'smartfeed_url_body.html')
);

make_jumpbox(append_sid("{$phpbb_root_path}viewforum.$phpEx"));

page_footer();

function base64_encode_urlsafe($input)
{
	// Thanks to phpBB forum user klapray for this logic for creating a "urlsafe" versions of base64_encode and _decode.
	return strtr(base64_encode($input), '+/=', '-_.');
}

function encrypt($data_input, $key)
{   

	// This function encrypts $data_input with the given $key. It uses the PHP Secure Communications Library, found
	// at http://phpseclib.sourceforge.net/. If mcrypt is compiled into PHP it will use that, otherwise it will use the functions 
	// from this library. This allows for maximum portability of Smartfeed. Thanks to Terrafrost at phpbb.com for the suggestion
	// to use this library!
	
	global $phpbb_root_path, $phpEx;
	static $des;
	
	if (!isset($des))
	{
		include($phpbb_root_path . "includes/Crypt/TripleDES.$phpEx");
    	$des = new Crypt_TripleDES();
	}

	$des->setKey($key);

	// Encrypt
	$encrypted_data = base64_encode_urlsafe($des->encrypt($data_input));
	return $encrypted_data;

}   

function check_all_parents($forum_id)
{

	// This function checks all parents for a given forum_id. If any of them do not have the f_list permission
	// the function returns false, meaning the forum should not be displayed because it has a parent that should
	// not be listed. Otherwise it returns true, indicating the forum can be listed.
	
	global $auth, $parent_array;
	
	$there_are_parents = true;
	$current_forum_id = $forum_id;
	$include_this_forum = true;
	
	while ($there_are_parents)
	{
	
		if ($parent_array[$current_forum_id] == 0) 	// No parent
		{
			$there_are_parents = false;
		}
		else
		{
			if ($auth->acl_get('f_list', $current_forum_id) == 1)
			{
				// So far so good
				$current_forum_id = $parent_array[$current_forum_id];
			}
			else
			{
				// Danger Will Robinson! No list permission exists for a parent of the requested forum, so this forum should not be shown
				$there_are_parents = false;
				$include_this_forum = false;
			}
		}
		
	}
	
	return $include_this_forum;
	
}

?>