<?php
/** 
*
* @package phpBB3
* @version $Id: smartfeed_url_new_key.php, v 2.2.27 Production 2015/01/14 mark@phpbbservices.com Exp $
* @copyright (c) Mark D. Hamill (mark@phpbbservices.com, http://phpbbservices.com) 
* @license http://opensource.org/licenses/gpl-2.0.php GNU Public License 
*
*/

/**
* @ignore
*/

// Written by Mark D. Hamill, mark@phpbbservices.com, http://phpbbservices.com
// This software is designed to work with phpBB Version 3.0.12

// This is a program that is designed to be called from an AJAX request. For a given user ID and valid
// session ID it creates a new value for user_smartfeed_key, stores it, then encrypts it and returns it 
// to the calling Javascript function in smartfeed_url_body.html. The encrypted password has two parts,
// one to be used if the IP authentication is not required, and one if it is. The two fields are delimited
// by a tilde, since one limitation of AJAX REST-like calls is that when used as a function it should 
// return only a single value.

define('IN_PHPBB', true);
$phpbb_root_path = (defined('PHPBB_ROOT_PATH')) ? PHPBB_ROOT_PATH : './';
$phpEx = substr(strrchr(__FILE__, '.'), 1);
include($phpbb_root_path . 'common.' . $phpEx);

// Start session management
$user->session_begin();
$auth->acl($user->data);

$user->setup('mods/smartfeed');
$board_url = generate_board_url() . '/';

// Check for the user_id argument
$user_id = request_var('u',0);
if ($user_id == 0)
{
	add_log('error', 'LOG_CONFIG_SMARTFEED_NO_U_ARGUMENT', $phpEx, $user->ip);
	exit_handler();
}

// Check for the sid argument
$sid = request_var('sid','NONE');
if ($sid == 'NONE')
{
	add_log('error', 'LOG_CONFIG_SMARTFEED_NO_SESSION_KEY', $phpEx, $user->ip);
	exit_handler();
}

// Check for the ip argument
$ip = request_var('ip','NONE');
if ($ip == 'NONE')
{
	add_log('error', 'LOG_CONFIG_SMARTFEED_NO_IP_ARGUMENT', $phpEx, $user->ip);
	exit_handler();
}

// Validate the session key is valid for the user. If it is valid, it is assumed that no hack attempt exists so it is okay
// to generate a new smartfeed key. Need to loop through the results rather than assume one result exists
// because multiple browsers may be in use for the same user at the same time.
$found_session = false;
$sql = 'SELECT session_id, username
		FROM ' . SESSIONS_TABLE . ' s, ' . USERS_TABLE . ' u
		WHERE session_user_id = user_id AND session_user_id = ' . intval($user_id) . " AND session_ip = '" . $user->ip . "'
		ORDER BY session_start";
$result = $db->sql_query($sql);
while ($row = $db->sql_fetchrow($result))
{
	if ($row['session_id'] == $sid)
	{
		$found_session = true;
		break;
	}
}

// If the session is not found, something is wrong so write a message to the error log. As this runs independently of smartfeed_url.php,
// no errors can be seen by the user.
if (!$found_session)
{
	$db->sql_freeresult($result);
	$sql = 'SELECT username
			FROM ' . USERS_TABLE . ' 
			WHERE user_id = ' . intval($user_id);
	$result = $db->sql_query($sql);
	$row = $db->sql_fetchrow($result);
	add_log('error', 'LOG_CONFIG_SMARTFEED_INVALID_SESSION_KEY', $sid, $row['username'], $phpEx);
	$db->sql_freeresult($result);
	exit_handler();
}
$db->sql_freeresult($result);

// Generate a new smartfeed encryption key.
$user_smartfeed_key = gen_rand_string(32);

// Store the key and the user IP. smartfeed.php will use the key to decrypt arguments provided in the URL to prove the feed request is legitimate.
// The stored IP will be checked against the current IP if that security policy is enforced
$sql = 'UPDATE ' . USERS_TABLE . "
	   SET user_smartfeed_key = '" . $db->sql_escape($user_smartfeed_key) . "', user_smartfeed_ip = '" . $ip . "'
	   WHERE user_id = " . (int) $user_id;
$result = $db->sql_query($sql);

add_log('admin', 'LOG_CONFIG_SMARTFEED_KEY_CHANGED', $row['username']);

exit_handler(); 

function base64_encode_urlsafe($input)
{
	// Thanks to phpBB forum user klapray for this logic for creating a "urlsafe" versions of base64_encode and _decode.
	return strtr(base64_encode($input), '+/=', '-_.');
}

function encrypt($data_input, $key)
{   

	// This function encrypts $data_input with the given $key. It uses the PHP Secure Communications Library, found
	// at http://phpseclib.sourceforge.net/. If mcrypt is compiled into PHP it will use that, otherwise it will use the functions 
	// from this library. This allows for maximum portability of Smartfeed. Thanks to Terrafrost at phpbb.com for the suggestion
	// to use this library!
	
	global $phpbb_root_path, $phpEx;
	static $des;
	
	if (!isset($des))
	{
		include($phpbb_root_path . "includes/Crypt/TripleDES.$phpEx");
    	$des = new Crypt_TripleDES();
	}

	$des->setKey($key);

	// Encrypt
	$encrypted_data = base64_encode_urlsafe($des->encrypt($data_input));
	return $encrypted_data;

}

?>